using System;
using System.Collections.Generic;
using System.Text;
using System.IO;


namespace DestCode
{
 	
    public class AdlerChecksum
    {
       
        #region
       
        public const uint AdlerBase  = 0xFFF1;
       
        public const uint AdlerStart = 0x0001;
       
        public const uint AdlerBuff = 0x0400;
       
        private uint m_unChecksumValue = 0;
        #endregion
        
        public uint ChecksumValue
        {
            get
            {
                return m_unChecksumValue;
            }
        }
		
        public bool MakeForBuff(byte[] bytesBuff, uint unAdlerCheckSum)
        {
            if (Object.Equals(bytesBuff, null))
            {
                m_unChecksumValue = 0;
                return false;
            }
            int nSize = bytesBuff.GetLength(0);
            if (nSize == 0)
            {
                m_unChecksumValue = 0;
                return false;
            }
            uint unSum1 = unAdlerCheckSum & 0xFFFF;
            uint unSum2 = (unAdlerCheckSum >> 16) & 0xFFFF;
            for (int i = 0; i < nSize; i++)
            {
                unSum1 = (unSum1 + bytesBuff[i]) % AdlerBase;
                unSum2 = (unSum1 + unSum2) % AdlerBase;
            }
            m_unChecksumValue = (unSum2 << 16) + unSum1;
            return true;
        }
        
        public bool MakeForBuff(byte[] bytesBuff)
        {
            return MakeForBuff(bytesBuff, AdlerStart);
        }
        
        public bool MakeForFile(string sPath)
        {
            try
            {
                if (!File.Exists(sPath))
                {
                    m_unChecksumValue = 0;
                    return false;
                }
                FileStream fs = new FileStream(sPath, FileMode.Open, FileAccess.Read);
                if (Object.Equals(fs, null))
                {
                    m_unChecksumValue = 0;
                    return false;
                }
                if (fs.Length == 0)
                {
                    m_unChecksumValue = 0;
                    return false;
                }
                m_unChecksumValue = AdlerStart;
                byte[] bytesBuff = new byte[AdlerBuff];
                for (uint i = 0; i < fs.Length; i++)
                {
                    uint index = i % AdlerBuff;
                    bytesBuff[index] = (byte)fs.ReadByte();
                    if ((index == AdlerBuff - 1) || (i == fs.Length - 1))
                    {
                        if (!MakeForBuff(bytesBuff, m_unChecksumValue))
                        {
                            m_unChecksumValue = 0;
                            return false;
                        }
                    }
                }
            }
            catch
            {
                m_unChecksumValue = 0;
                return false;
            }
            return true;
        }
       
        public override bool Equals(object obj)
        {
            if (obj == null)
                return false;
            if (this.GetType() != obj.GetType())
                return false;
            AdlerChecksum other = (AdlerChecksum)obj;
            return (this.ChecksumValue == other.ChecksumValue);
        }
        
       
        public static bool operator ==(AdlerChecksum objA, AdlerChecksum objB)
        {
            if (Object.Equals(objA, null) && Object.Equals(objB, null)) return true;
            if (Object.Equals(objA, null) || Object.Equals(objB, null)) return false;
            return objA.Equals(objB);
        }
        
        public static bool operator !=(AdlerChecksum objA, AdlerChecksum objB)
        {
            return !(objA == objB);
        }
       
        public override int GetHashCode()
        {
            return ChecksumValue.GetHashCode();
        }
       
        public override string ToString()
        {
            if (ChecksumValue != 0)
                return ChecksumValue.ToString();
            return "Unknown";
        }
    }
}
